{
   Written by David Gwillim  5 April 1990

   Inserts the contents of a data file into the header of an EXE file
   after the relocation table. The old header size (in paragraphs) is
   stored in the checksum word (at offset 0012h) of the output file.
   The header of the output file is updated to show the new header and
   overall EXE file size.

   The data file inserted can theoretically be up to 1.4 megabytes
   (65535 paragraphs) long less the size of the original EXE header. If
   the data file is not an even number of paragraphs long it is rounded
   up the next paragraph boundary with 0-bytes when it is inserted, as
   the EXE header has to be an even number of paragraphs.

   This provides a simple way to store information inside an EXE file
   that can be accessed at an easily determinable offset.
}

{$I-,M 4096,65535,65535}


uses
   Dos;

type
   ExeHdr = record
      Signature : word;
      LastPageBytes : word;
      FilePages : word;
      RelocationCount : word;
      HeaderParas : word;
      MinAllocParas : word;
      MaxAllocParas : word;
      StackSegmentOffset : word;
      StackPointerInit : word;
      Checksum : word;
      InstructionPointerInit : word;
      CodeSegmentOffset : word;
      RelocationTablePointer : word;
      OverlayNumber : word;
      Padding : array[1..2] of word;
   end;
   Arr2 = array[1..2] of char;
   Buffer = array[1..3750*16] of byte;

const
   ParaFill : array[1..15] of char = #0#0#0#0#0#0#0#0#0#0#0#0#0#0#0;

var
   inF : file;
   inName : PathStr;
   outF : file;
   outName : PathStr;
   datF : file;
   datName : PathStr;
   datSz : longint;
   datParas : word;
   B : ^Buffer;
   EH : ExeHdr;
   HdrBytes : longint;
   n : word;
   BytesRead : word;
   ReadSize : word;

{
Ŀ
 ShowHeader 

}
procedure ShowHeader;
begin
   with EH do
   begin
      writeln('Signature ................ ',Arr2(Signature));
      writeln('LastPageBytes ............ ',LastPageBytes);
      writeln('FilePages ................ ',FilePages);
      writeln('RelocationCount .......... ',RelocationCount);
      writeln('HeaderParas .............. ',HeaderParas);
      writeln('MinAllocParas ............ ',MinAllocParas);
      writeln('MaxAllocParas ............ ',MaxAllocParas);
      writeln('StackSegmentOffset ....... ',StackSegmentOffset);
      writeln('StackPointerInit ......... ',StackPointerInit);
      writeln('Checksum ................. ',Checksum);
      writeln('InstructionPointerInit ... ',InstructionPointerInit);
      writeln('CodeSegmentOffset ........ ',CodeSegmentOffset);
      writeln('RelocationTablePointer ... ',RelocationTablePointer);
      writeln('OverlayNumber ............ ',OverlayNumber);
   end;
end;

{
Ŀ
 ShowUsage 

}
procedure ShowUsage;
begin
   writeln;
   writeln('ͻ');
   writeln('            Data Insertion Utility for EXE files           ');
   writeln('                 Version 1.0   5 April 1990                ');
   writeln('            Copyright (C) 1990 by David Gwillim            ');
   writeln('Ķ');
   writeln('   USAGE:                                                  ');
   writeln('       EXEDAT InputExeName DataFileName OutputExeName      ');
   writeln('ͼ');
   Halt(1);
end;

{
Ŀ
 Abort 

}
procedure Abort(Msg : PathStr);
begin
   writeln;
   writeln(#7' ERROR: ',Msg);
   Halt(1);
end;

begin
   if ParamCount < 3 then
      ShowUsage;

   inName  := FExpand(ParamStr(1));
   datName := FExpand(ParamStr(2));
   outName := FExpand(ParamStr(3));

   writeln;
   assign(inF,inName);
   reset(inF,1);
   if IOresult <> 0 then
      Abort('Input file not found: '+inName);

   writeln(' Reading EXE header for file: ',inName);

   BlockRead(inF,EH,SizeOf(EH));
   if Arr2(EH.Signature) <> 'MZ' then
   begin
      writeln(#7'ERROR: Input file is not a valid EXE file');
      Halt(1);
   end;

   assign(datF,datName);
   {$I-}
   reset(datF,1);
   {$I+}
   if IOresult <> 0 then
      Abort('Data file not found: '+datName);

   datSz := FileSize(datF);
   datParas := datSz div 16;
   if datSz mod 16 <> 0 then
      Inc(datParas);

   writeln(' Saving old header paragraph count in new header checksum');

   HdrBytes := (EH.HeaderParas * 16) - SizeOf(ExeHdr);
   with EH do
   begin
      { save the old header size in the unused checksum }
      Checksum := HeaderParas;
      { add in the paragraph count of the inserted data file }
      HeaderParas := HeaderParas + datParas;
      FilePages := FilePages + (longint(datParas) * 16) div 512;
      LastPageBytes := LastPageBytes + (longint(datParas) * 16) mod 512;
      if LastPageBytes > 512 then
      begin
         Inc(FilePages);
         LastPageBytes := LastPageBytes mod 512;
      end;
   end;
   writeln(' Creating new EXE file: ',outName);

   assign(outF,outName);
   rewrite(outF,1);
   if IOresult <> 0 then
      Abort('Cannot create the file: '+outName);

   writeln(' Writing updated header information to file: ',outName);
   BlockWrite(outF,EH,SizeOf(EH));
   if IOresult <> 0 then
      Abort('Insufficient disk space');

   New(B);

   writeln(' Transferring relocation table');
   if HdrBytes > SizeOf(Buffer) then
      ReadSize := SizeOf(Buffer)
   else
      ReadSize := HdrBytes;
   repeat
      BlockRead(inF,B^,ReadSize,BytesRead);
      BlockWrite(outF,B^,BytesRead);
      if IOresult <> 0 then
         Abort('Insufficient disk space');
      Dec(HdrBytes,BytesRead);
      if HdrBytes < ReadSize then
         ReadSize := HdrBytes;
   until HdrBytes <= 0;

   writeln(' Inserting data from file: ',datName);
   repeat
      BlockRead(datF,B^,SizeOf(Buffer),BytesRead);
      BlockWrite(outF,B^,BytesRead);
      if IOresult <> 0 then
         Abort('Insufficient disk space');
   until BytesRead <> SizeOf(Buffer);
   if datSz mod 16 <> 0 then
      BlockWrite(OutF,ParaFill,datParas*16 - datSz);
   if IOresult <> 0 then
      Abort('Insufficient disk space');
   close(datF);
   if IOresult <> 0 then
      Abort('Insufficient disk space');

   writeln(' Transferring executable image from file: ',inName);
   repeat
      BlockRead(Inf,B^,SizeOf(Buffer),BytesRead);
      BlockWrite(OutF,B^,BytesRead);
      if IOresult <> 0 then
         Abort('Insufficient disk space');
   until BytesRead <> SizeOf(Buffer);

   close(inF);
   close(outF);
   if IOresult <> 0 then
      Abort('Insufficient disk space');

   Dispose(B);
   writeln(' Data insertion completed.');
end.
